from django.db import models
from django import forms
from django.utils.translation import ugettext_lazy as _

from .base import detect_backend

__all__ = ('EmbedVideoField', 'EmbedVideoFormField')


class EmbedVideoField(models.URLField):
    def formfield(self, **kwargs):
        defaults = {'form_class': EmbedVideoFormField}
        defaults.update(kwargs)
        return super(EmbedVideoField, self).formfield(**defaults)

    def south_field_triple(self):
        from south.modelsinspector import introspector
        cls_name = '%s.%s' % (
            self.__class__.__module__,
            self.__class__.__name__
        )
        args, kwargs = introspector(self)
        return (cls_name, args, kwargs)


class EmbedVideoFormField(forms.URLField):
    def validate(self, url):
        super(EmbedVideoFormField, self).validate(url)

        try:
            detect_backend(url)
        except:
            raise forms.ValidationError(_(u'URL could not be recognized.'))

        return url



import os

from setuptools import setup, find_packages

setup(
    name = 'django-embed-video',
    packages = find_packages(),
    version = '0.0.1',
    author = 'Juda Kaleta',
    author_email = 'juda.kaleta@gmail.com',
    url = 'https://github.com/yetty/django-embed-video',
    description = 'Django template tags for YouTube and Vimeo',
    long_description = open('README').read(),
    classifiers = [
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords = ['youtube', 'vimeo', 'video'],
    test_suite = 'embed_video.tests.tests',
)



import re

DETECT_YOUTUBE = re.compile(
    '^(http(s)?://(www\.)?)?youtu(\.?)be(\.com)?.*', re.I
)
DETECT_VIMEO = re.compile('^(http(s)?://(www\.)?)?vimeo\.com.*', re.I)


class UnknownBackendException(Exception):
    pass


def detect_backend(url):
    if DETECT_YOUTUBE.match(url):
        return YoutubeBackend(url)
    elif DETECT_VIMEO.match(url):
        return VimeoBackend(url)
    else:
        raise UnknownBackendException


class VideoBackend(object):
    def __init__(self, url):
        self._url = url

        self.code = self.get_code()
        self.url = self.get_url()
        self.thumbnail = self.get_thumbnail_url()

    def get_code(self):
        match = self.re_code.search(self._url)
        if match:
            return match.group('code')

    def get_url(self):
        return self.pattern_url % self.code

    def get_thumbnail_url(self):
        return self.pattern_thumbnail_url % self.code


class YoutubeBackend(VideoBackend):
    re_code = re.compile(
        'youtu(?:be\.com/watch\?v=|\.be/)(?P<code>[\w-]*)(&(amp;)?[\w\?=]*)?',
        re.I
    )
    pattern_url = 'http://www.youtube.com/embed/%s?wmode=opaque'
    pattern_thumbnail_url = 'http://img.youtube.com/vi/%s/hqdefault.jpg'


class VimeoBackend(VideoBackend):
    re_code = re.compile('vimeo\.com/(?P<code>[0-9]+)', re.I)
    pattern_url = 'http://player.vimeo.com/video/%s'

    def get_thumbnail_url(self):
        pass  # not implemented
